/*********************************************************************
 * This program will allow on to turn a given port on and off at a user
 * defined rate.  The code was initially developed by Dingo_aus, 7 January 2009
 * email: dingo_aus [at] internode <dot> on /dot/ net
 * From http://www.avrfreaks.net/wiki/index.php/Documentation:LinuxGPIO#gpio_framework (Note: This link is now broken.)
 * Created in AVR32 Studio (version 2.0.2) running on Ubuntu 8.04
 * Modified by Mark A. Yoder, 21-July-2011
 * Refactored and further modified by Walter Schilling, Summer 2012 / Winter 2013-2014.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <poll.h>
#include <signal.h>	// Defines signal-handling functions (i.e. trap Ctrl-C)
#include <stdint.h>
#include "gpioInterface.h"

#define POLL_TIMEOUT (1 * 1000) /* 1 second */
#define MAX_BUF 64

// create a variable to keep track of the port that is to be opened as a string.
static int keepgoing = 1;	// Set to 0 when ctrl-c is pressed

/*******************************************************************************
 * This method will read the given pin and perform processing on it.
 *
 * @param FILE *fp - This is a pointer to the io device driver which will be read to determine the state of the pin.
 ******************************************************************************/
int processPin(int gpio_fd, int outputPin, uint32_t gpioInputPort)
{
	struct pollfd fdset[2];
	int timeout, rc;
	int nfds = 2;
	char buf[MAX_BUF];
	int len;

	//create a variable to store whether we are sending a '1' or a '0'
	char set_value[5];
	char prevState; 

	prevState = '?';
	timeout = POLL_TIMEOUT;

	while (keepgoing) {
		memset((void*)fdset, 0, sizeof(fdset));

		fdset[0].fd = STDIN_FILENO;
		fdset[0].events = POLLIN;
      
		fdset[1].fd = gpio_fd;
		fdset[1].events = POLLPRI;

		rc = poll(fdset, nfds, timeout);      // Sits and waits for an interrupt.

		if (rc < 0) {
			printf("\npoll() failed!  Something is going on.  Is the program exiting?\n");
			return -1;
		}
      
		if (rc == 0) {
			printf(".");
		}
            
		if (fdset[1].revents & POLLPRI) {
			lseek(fdset[1].fd, 0, SEEK_SET);  // Read from the start of the file
			len = read(fdset[1].fd, buf, MAX_BUF);
			printf("\npoll() GPIO %d interrupt occurred, value=%c, len=%d\n", gpioInputPort, buf[0], len);

			if (buf[0]!=prevState)
			{
				if (buf[0]=='0')
				{
					printf("The button is pressed.\n");

					//Write our value of "0" to the file 
					 gpio_set_value(outputPin, 0);
				}
				else
				{
					printf("The button is not pressed.\n");

					//Write our value of "1" to the file 
					gpio_set_value(outputPin, 1);
				}
				prevState = buf[0];
			}
		}
		if (fdset[0].revents & POLLIN) {
			(void)read(fdset[0].fd, buf, 1);
			printf("\npoll() stdin read 0x%2.2X\n", (unsigned int) buf[0]);
		}

		fflush(stdout);
	}
}

/****************************************************************
 * signal_handler
 ****************************************************************/
// Callback called when SIGINT is sent to the process (Ctrl-C)
void signal_handler(int sig)
{
	printf( "Ctrl-C pressed, cleaning up and exiting..\n" );
	keepgoing = 0;
}



/****************************************************************
 * Main
 ****************************************************************/
int main(int argc, char **argv, char **envp)
{
	/* This variable is a file pointer to the file interface for the GPIO device driver
	   which controls the input pin. */ 
	uint32_t gpio_fd; 			// This is the file ID for the input file.
	uint8_t gpioInputPort;   	// This is the GPIO port that is to be used for input into this program.
	uint8_t gpioOutputPort;		//  This is the GPIO port that is to be used for output from this program. 
	char ioPort[56];        	// This is the IO port that is to be referenced.
	
	if (argc < 3) {
		printf("Usage: %s <input port number> <output port number>\n\n", argv[0]);
		printf("Waits for a change in the GPIO pin voltage level or input on stdin\n");
		exit(-1);
	}
	// Set the signal callback for Ctrl-C
	signal(SIGINT, signal_handler);

	// Convert the input into the appropriate parameters. 
    gpioInputPort = atoi(argv[1]);
    gpioOutputPort = atoi(argv[2]);

	printf("\n*********************************\n"
		"*  Welcome to LED follower program Demo program. *\n"
		"*  ....Reading gpio %d Controlling GPIO %d       *\n"
		"**********************************\n", gpioInputPort, gpioOutputPort);

	// Setup the input port.
	(void)gpio_export(gpioInputPort);
	(void)gpio_set_dir(gpioInputPort, 0);
	(void)gpio_set_edge(gpioInputPort, GPIO_BOTH_EDGES);  // Both indicates that an interrupt will fire on oth a rising and falling edge.
	gpio_fd = gpio_fd_open(gpioInputPort);

	// Setup the output port.
	(void)gpio_export(gpioOutputPort);
	(void)gpio_set_dir(gpioOutputPort, 1);
	(void)gpio_fd_open(gpioOutputPort);

	// Start the process pin routine which will handle processing the pin.
	processPin(gpio_fd, gpioOutputPort, gpioInputPort);

	//***********************************************************************
 	// cleanup the executing system
	// Close the pins
	gpio_fd_close(gpio_fd);
	
	// Unexport the pins.
	gpio_unexport(gpioInputPort);
	gpio_unexport(gpioOutputPort);
	
	printf("Goodbye!\n");

	return 0;
}


